# Copyright (c) 2016, LE GOFF Vincent
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.

# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# * Neither the name of ytranslate nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import wx

from accesspanel.extensions.base import BaseExtension

class CommandHistory(BaseExtension):

    """Implement a command history.

    Features of command history can be set using attributes.  Here is the list of attributes, see below for an example:
        lock: what key to press to enter/leave lock mode (tuple)
        up: what key to press to go up in the history (tuple)
        down: what key to press to go down in the history (tuple)

    The attributes needing a key must be described as a tuple
    (modifiers, key) where both modifiers and keys are wxPython's key
    codes.  For instance:

    >>> import wx
    >>> from accesspanel import AccessPanel
    >>> class MyAccessPanel(AccessPanel):
    ...     def __init__(self, parent):
    ...         AccessPanel.__init__(self, parent, history=True)
    ...         # Configure the history
    ...         history = self.extensions["history"]
    ...         history.lock = (wx.MOD_NONE, wx.WXK_ESCAPE)
    ...         history.up = (wx.MOD_CONTROL, wx.WXK_UP)
    ...         history.down = (wx.MOD_CONTROL, wx.WXK_DOWN)

    Default values:
        lock: escape
        up: Ctrl + Up arrow
        down: Ctrl + Down arrow

    If you with to modify these default values, see the example above.
    If you wish to remove the features (for instance, the lock), set
    it to None.

    """

    def __init__(self, panel):
        BaseExtension.__init__(self, panel)
        self.commands = []
        self.position = -1
        self.locking = False

        # Features that can be set in the AccessPanel
        self.lock = (wx.MOD_NONE, wx.WXK_ESCAPE)
        self.up = (wx.MOD_CONTROL, wx.WXK_UP)
        self.down = (wx.MOD_CONTROL, wx.WXK_DOWN)

    def OnInput(self, text):
        """A command is sent, add it into the history."""
        self.position = -1
        for command in text.splitlines():
            if self.commands and command == self.commands[-1]:
                continue

            self.commands.append(command)

        return text

    def OnKeyDown(self, modifiers, key):
        """Add the keyboard shortcuts for the history navigation."""
        skip = False
        shortcut = (modifiers, key)
        lock_up = (wx.MOD_NONE, wx.WXK_UP)
        lock_down = (wx.MOD_NONE, wx.WXK_DOWN)

        if shortcut == self.lock:
            self.locking = not self.locking
        elif self.locking and shortcut == lock_up:
            self.GoUp()
        elif self.locking and shortcut == lock_down:
            self.GoDown()
        elif shortcut == self.up:
            self.GoUp()
        elif shortcut == self.down:
            self.GoDown()
        else:
            skip = True

        return skip

    def GoUp(self):
        """Go up in the history."""
        if self.position < 0:
            self.position = len(self.commands)
        elif self.position == 0:
            return

        self.position -= 1
        try:
            text = self.commands[self.position]
        except IndexError:
            pass
        else:
            self.panel.input = text

    def GoDown(self):
        """Go down in the history."""
        if self.position < 0:
            return
        elif self.position >= len(self.commands) - 1:
            self.position = -1
            text = ""
        else:
            self.position += 1
            try:
                text = self.commands[self.position]
            except IndexError:
                return

        self.panel.input = text



# Copyright (c) 2016, LE GOFF Vincent
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.

# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# * Neither the name of ytranslate nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Module containing the base class for extensions."""

class BaseExtension:

    """Base class for extensions.

    Extensions can had features to the AccessPanel.  They can override
    features of the AccessPanel, like its handling of keyboard
    shortcuts, in a way defined by the AccessPanel itself.  Extensions
    aren't mixin classes, the AccessPanel doesn't inherit from them,
    it implements them in a given list (see the AccessPanel class).

    """

    def __init__(self, panel):
        self.panel = panel

    def OnInput(self, text):
        """Override this method to change the way OnInput behaves.

        This method should return the text, that can be altered.  The
        extensions' 'OnInput' method will be called before the
        AccessPanel's implementation, which allows modifications of
        the text.

        """
        return text

    def OnMessage(self, text):
        """Alter the message sent to the AccessPanel.

        The given message is sent to the AccessPanel after the
        extensions have modified it.  This method should also return
        the modified text.

        """
        return text

    def PostMessage(self, message):
        """The message has been updated successfully>"""
        pass

    def OnKeyDown(self, modifiers, key):
        """Add keyboard handling for this extension.

        This method should return whether this event should be skipped
        or not.  If it is skipped, then the key press will stop at
        this extension.  If not, it is passed along other extensions
        and, ultimately, to the AccessPanel itself if no other extension
        requires it to be skipped.

        """
        return True



# Copyright (c) 2016, LE GOFF Vincent
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.

# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# * Neither the name of ytranslate nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Module containing the LockInput extension."""

import wx

from accesspanel.extensions.base import BaseExtension

class LockInput(BaseExtension):

    """Implement a lock-in-input mode.

    In this mode, the user cannot use tab or shift-tab to leave the
    input field.  The cases in which the lock is applied can be changed
    through the extension's settings.

    Behavior of the extension can be altered through attributes:
        empty: the lock will be active unless the input is empty

    >>> import wx
    >>> from accesspanel import AccessPanel
    >>> class MyAccessPanel(AccessPanel):
    ...     def __init__(self, parent):
    ...         AccessPanel.__init__(self, parent, lock_input=True)
    ...         # Configure the lock
    ...         lock = self.extensions["lock_input"]
    ...         lock.empty = True

    Default values:
        input: False

    If you with to modify these default values, see the example above.

    """

    def __init__(self, panel):
        BaseExtension.__init__(self, panel)

        # Features that can be set in the AccessPanel
        self.empty = False

    def OnKeyDown(self, modifiers, key):
        """Prevent changing focus with tab/shift-tab."""
        skip = True
        if modifiers in (wx.MOD_NONE, wx.MOD_SHIFT) and key == wx.WXK_TAB:
            if not self.empty:
                skip = False
            elif self.panel.input:
                skip = False

        return skip



# Copyright (c) 2016, LE GOFF Vincent
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.

# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# * Neither the name of ytranslate nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Module containing the AccessPanel class.

The AccessPanel is a child of wx.Panel.  It behaves like an ordinary
panel with a multi-line text field taking all available room.  This
output field can serve as input:

If the user types in the output field, the cursor is moved to the
bottom of the text field where he/she can type.  The AccessPanel is
like a big read-only text area, except for the last line(s).

Additional features:
1.  Command history
    The AccessPanel supports a command history, meaning it will
    remember what command has been entered and will allow to navigate
    through the command history using CTRL + arrow keys.
    You can activate it by setting the 'history' argument to True
    when creating an AccessPanel.
2.  Lock input
    The AccessPanel can lock the user in the input, meaning the user
    cannot leave the input field using tab or shift-tab.
3.  ANSI support
    ANSI codes are displayed in the AccessPanel.  See the 'ANSI'
    extension for more details.

"""

from collections import OrderedDict

import wx

import extensions

## Constants
# Navigation keys (arrows, home/end, pageUp/pageDown...)
NAV_KEYS = set()
NAV_KEYS.add(wx.WXK_UP)
NAV_KEYS.add(wx.WXK_DOWN)
NAV_KEYS.add(wx.WXK_LEFT)
NAV_KEYS.add(wx.WXK_RIGHT)
NAV_KEYS.add(wx.WXK_HOME)
NAV_KEYS.add(wx.WXK_END)
NAV_KEYS.add(wx.WXK_PAGEUP)
NAV_KEYS.add(wx.WXK_PAGEDOWN)
NAV_KEYS.add(wx.WXK_NUMLOCK)
NAV_KEYS.add(wx.WXK_PAUSE)
NAV_KEYS.add(wx.WXK_CAPITAL)
NAV_KEYS.add(wx.WXK_SCROLL)
NAV_KEYS.add(wx.WXK_WINDOWS_LEFT)
NAV_KEYS.add(wx.WXK_WINDOWS_RIGHT)
NAV_KEYS.add(wx.WXK_CONTROL)
NAV_KEYS.add(wx.WXK_SHIFT)

# Event definition
myEVT_MESSAGE = wx.NewEventType()
EVT_MESSAGE = wx.PyEventBinder(myEVT_MESSAGE, 1)

class MessageEvent(wx.PyCommandEvent):

    """Event when a message is received."""

    def __init__(self, etype, eid, value=None):
        wx.PyCommandEvent.__init__(self, etype, eid)
        self._value = value

    def GetValue(self):
        """Return the event's value."""
        return self._value


## AccessPanel class
class AccessPanel(wx.Panel):

    """Access panel with a text field (TextCtrl) in it.

    Constructor:
        parent: the parent window where the panel is defined.
        history (default False): activate command history.
        lock_input (default False): activate the lock in input.
        ansi (default False): activate the ANSI extension.

    Example:
    >>> import wx
    >>> from accesspanel import AccessPanel
    >>> class MyAccessPanel(AccessPanel):
    ...     def __init__(self, parent):
    ...         AccessPanel.__init__(self, parent, history=True)
    ...
    ...     def OnInput(self, text):
    ...         print "I received", input
    >>>
    >>> class MainWindow(wx.Frame):
    ...     def __init__(self):
    ...         wx.Frame.__init__(self, None)
    ...         self.panel = MyAccessPanel(self)
    ...         # Write something in the text field
    ...         self.panel.send("This is a text\nThat you shouldn't edit.")
    ...         self.Maximize()
    ...         self.show()

    Attributes and properties:
        output: The output text field (the TextCtrl)
        input: the text contained in the lines allowed for editing

    Methods:
        IsEditing: is the cursor in the editing section?
        OnInput: text is sent by the user pressing RETURN.
        ClearInput: the input text is cleared.
        Send: send text to the output field (it will added in the output).

    """

    def __init__(self, parent, history=False, lock_input=False,
            ansi=False):
        super(AccessPanel, self).__init__(parent)
        self.editing_pos = 0
        self.extensions = OrderedDict()

        # Build the extensions
        if history:
            extension = extensions.CommandHistory(self)
            self.extensions["history"] = extension
        if lock_input:
            extension = extensions.LockInput(self)
            self.extensions["lock_input"] = extension
        if ansi:
            extension = extensions.ANSI(self)
            self.extensions["ANSI"] = extension

        # Window design
        sizer = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(sizer)

        # Output field
        output = wx.TextCtrl(self, size=(600, 400),
                style=wx.TE_MULTILINE | wx.TE_RICH2)
        self.output = output

        # Add the output field in the sizer
        sizer.Add(output, 1, wx.EXPAND)

        # Event handler
        self.Bind(EVT_MESSAGE, self.OnMessage)
        output.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)

        # Panel design
        sizer.Fit(self)

    def _get_input(self):
        """Return the text being edited.

        This text should be between the editing position (editing_pos)
        and the end of the output field.

        """
        return self.output.GetRange(
                self.editing_pos, self.output.GetLastPosition())

    def _set_input(self, text):
        """Set the text in the input field."""
        self.ClearInput()
        self.output.AppendText(text)
        self.output.SetInsertionPoint(self.output.GetInsertionPoint())
    input = property(_get_input, _set_input)

    def IsEditing(self, beyond_one=False):
        """Return True if editing, False otherwise.

        We consider the user is editing if the cursor is in the input area.

        The 'beyond_one' parameter can be set to True if we want to
        test whether the cursor is at least one character ahead in
        the input field.

        """
        pos = self.output.GetInsertionPoint()
        if beyond_one:
            pos -= 1

        return pos < self.editing_pos

    def ClearInput(self):
        """Clear the input text."""
        self.output.Remove(
                self.editing_pos, self.output.GetLastPosition())
        self.editing_pos = self.output.GetLastPosition()

    def OnInput(self, message):
        """A message has been sent by pressing RETURN.

        This method should be overridden in child classes.

        """
        pass

    def OnMessage(self, e):
        """A message is received and should be displayed in the output field.

        This method is directly called in answer to the EVT_MESSAGE.
        It displays the received text in the window, being careful to
        put the cursor where it was before, with the typed text in
        the input field.

        """
        self.output.Freeze()
        message = e.GetValue()

        # Normalize new lines
        message = "\r\n".join(message.splitlines())

        # Modify the text based on extensions
        for extension in self.extensions.values():
            message = extension.OnMessage(message)
            if not message:
                return

        if not message.endswith("\r\n"):
            message += "\r\n"

        pos = self.output.GetInsertionPoint()

        # Get the text before the editing line
        output = self.output.GetRange(0, self.editing_pos)
        input = self.input

        # Clears the output field and pastes the text back in
        self.output.ChangeValue(output + message)

        # If the cursor is beyond the editing position
        if pos >= self.editing_pos:
            pos += len(message)

        # We have added some text, the editing position should be
        # at the end of the text
        self.editing_pos = self.output.GetLastPosition()
        self.output.AppendText(input)
        self.output.SetInsertionPoint(pos)

        # Call the extensions' PostMessage
        for extension in self.extensions.values():
            extension.PostMessage(message)

        self.output.Thaw()

    def Send(self, message):
        """Create an event to send the message to the window."""
        evt = MessageEvent(myEVT_MESSAGE, -1, message)
        wx.PostEvent(self, evt)

    def OnKeyDown(self, e):
        """A key is pressed in the output field."""
        skip = True
        pos = self.output.GetInsertionPoint()
        input = self.input
        modifiers = e.GetModifiers()
        key = e.GetUnicodeKey()
        if not key:
            key = e.GetKeyCode()

        # If the user has pressed RETURN
        if key == wx.WXK_RETURN and modifiers == 0:
            self.ClearInput()

            # Call the extensions' 'OnInput'
            for extension in self.extensions.values():
                input = extension.OnInput(input)
                if not input:
                    break

            self.OnInput(input)
            skip = False

        # If we press a letter before the input area, move it back there
        if pos < self.editing_pos and modifiers in (wx.MOD_NONE, wx.MOD_SHIFT):
            if key not in NAV_KEYS:
                self.output.SetInsertionPoint(self.output.GetLastPosition())
                pos = self.output.GetInsertionPoint()

        # If we press backspace out of the input area
        if key == wx.WXK_BACK and modifiers == 0 and self.IsEditing(True):
            skip = False

        # Call the extensions' 'OnKeyDown' method
        if skip:
            for extension in self.extensions.values():
                skip = extension.OnKeyDown(modifiers, key)
                if not skip:
                    return

        if skip:
            e.Skip()



# Copyright (c) 2016, LE GOFF Vincent
# All rights reserved.

# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice, this
#   list of conditions and the following disclaimer.

# * Redistributions in binary form must reproduce the above copyright notice,
#   this list of conditions and the following disclaimer in the documentation
#   and/or other materials provided with the distribution.

# * Neither the name of ytranslate nor the names of its
#   contributors may be used to endorse or promote products derived from
#   this software without specific prior written permission.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import re

import wx
import wx.lib.colourdb

from accesspanel.extensions.base import BaseExtension

## Constants
# Regular expressions to capture ANSI codes
RE_ANSI = re.compile(r'\x1b\[([^m]*)m', re.UNICODE)
RE_CODE = re.compile(r"^(?:(\d+)(?:\;(\d+)(?:\;(\d+))?)?)?$", re.UNICODE)

# Brightness
NORMAL = 1
BRIGHT = 2
UNDERLINE = 3

class ANSI(BaseExtension):

    """Manage the ANSI codes to have color in the text.

    The AccessPanel can support color codes. This extension allows
    to write ANSI color codes and put the text in correct formatting.

    Brief reminder:
        ANSI codes are placed in the text, between a '\x1b[' sequence
        and a final 'm'. In between are numbers separated with a
        semicolon.  The first number represents the level of
        brightness (0 for normal, 1 for bright, 4 for underline,
        7 for negative).  The second number represents the foreground
        color and should be between 30 and 37 (see the list below).
        The third number represents the background color and should be
        between 40 and 47 (the same list of color applies).

    List of colors:
        | Color      | Foreground | Background |
        | Black      | 30         | 40         |
        | Red        | 31         | 41         |
        | Green      | 32         | 42         |
        | Yellow     | 33         | 43         |
        | Blue       | 34         | 44         |
        | Magenta    | 35         | 45         |
        | Cyan       | 36         | 46         |
        | White      | 37         | 47         |

    Examples:
        "\x1b[0;31;47m" means red on white.
        "\x1b[1;33m" means bright yellow on default background.
        "\x1b[4;36m" means underline cyan on default background.
        "\x1b[0m" means back to default colors.

    """

    def __init__(self, panel):
        BaseExtension.__init__(self, panel)
        wx.lib.colourdb.updateColourDB()
        self.brightness = NORMAL
        self.foreground = wx.BLACK
        self.background = wx.WHITE
        self.default_foreground = wx.BLACK
        self.default_background = wx.WHITE
        self.modifiers = []

        # Color codes
        self.normal_colors = {
            40: wx.NamedColour("dark grey"),
            41: wx.RED,
            42: wx.GREEN,
            43: wx.YELLOW,
            44: wx.BLUE,
            45: wx.NamedColour("magenta"),
            46: wx.CYAN,
            47: wx.WHITE,
        }

        self.bright_colors = {
            40: wx.NamedColour("grey"),
            41: wx.NamedColour("deep pink"),
            42: wx.NamedColour("light green"),
            43: wx.NamedColour("light yellow"),
            44: wx.NamedColour("light blue"),
            45: wx.NamedColour("light magenta"),
            46: wx.NamedColour("light cyan"),
            47: wx.WHITE,
        }

        self.dark_colors = {
            40: wx.BLACK,
            41: wx.NamedColour("dark red"),
            42: wx.NamedColour("dark green"),
            43: wx.NamedColour("orange"),
            44: wx.NamedColour("dark blue"),
            45: wx.NamedColour("dark magenta"),
            46: wx.NamedColour("dark cyan"),
            47: wx.WHITE,
        }

    def OnMessage(self, message):
        """Interpret the ANSI codes."""
        point = self.panel.editing_pos
        pos = []

        # Browse through the ANSI codes
        for match in reversed(list(RE_ANSI.finditer(message))):
            code = match.group(1)

            # Extract the brightness, foreground and background
            codes = RE_CODE.search(code)
            if codes:
                brightness, foreground, background = codes.groups()

                # All 3 variables can be None
                if brightness == 1:
                    brightness = BRIGHT
                elif brightness == 4:
                    brightness = UNDERLINE
                else:
                    brightness = NORMAL

                # Extract the foreground color
                if foreground:
                    foreground = int(foreground)
                    if brightness == NORMAL:
                        foreground = self.normal_colors.get(foreground + 10)
                    elif brightness == BRIGHT:
                        foreground = self.bright_colors.get(foreground + 10)
                    elif brightness == UNDERLINE:
                        foreground = self.dark_colors.get(foreground + 10)

                # Extract the background color
                if background:
                    background = int(background)
                    if brightness == NORMAL:
                        background = self.normal_colors.get(background)
                    elif brightness == BRIGHT:
                        background = self.bright_colors.get(background)
                    elif brightness == UNDERLINE:
                        background = self.dark_colors.get(background)

                if foreground is None:
                    foreground = self.default_foreground

                if brightness:
                    self.brightness = brightness

                if background is None:
                    if self.brightness == BRIGHT:
                        background = wx.BLACK
                    elif self.brightness == UNDERLINE:
                        background = wx.YELLOW
                    else:
                        background = self.default_background

                start = match.start()
                end = match.end()
                pos.append((start, end, foreground, background))

        begin_tag = True
        updated_pos = 0
        last_mod = None
        for start, end, foreground, background in reversed(pos):
            if begin_tag:
                begin_tag = False
            else:
                eol = message.count("\r", 0, start - 1)
                begin_tag = True
                real_start, m_foreground, m_background = last_mod
                real_start += point
                real_start -= eol
                real_end = start - updated_pos + point
                real_end -= eol
                self.modifiers.append((real_start, real_end, m_foreground,
                        m_background))

                # If it's not the default color, mark an open tag
                if foreground != self.default_foreground or \
                        background != self.default_background:
                    begin_tag = False

            last_mod = (start - updated_pos, foreground, background)
            updated_pos += end - start

        # Remove the ANSI codes from the message
        for start, end, foreground, background in pos:
            message = message[:start] + message[end:]

        return message

    def PostMessage(self, message):
        for start, end, foreground, background in self.modifiers:
            range = self.panel.output.GetRange(start, end)
            self.panel.output.SetStyle(start, end, wx.TextAttr(
                    foreground, background))



from setuptools import setup, find_packages

DESCRIPTION = """
AccessPanel is a module to enhance accessibility though a set of
classes derived from wx.
""".strip()

setup(
    name = "accesspanel",
    version = "0.11",
    packages = find_packages(),
    install_requires = [],
    description = DESCRIPTION,
    author = 'Vincent Le Goff',
    author_email = 'vincent.legoff.srs@gmail.com',
    url = 'https://github.com/vlegoff/accesspanel',
    keywords = ['accessibility'],
)
